package solutions.bitbadger.documents

import org.junit.jupiter.api.DisplayName
import solutions.bitbadger.documents.query.Count
import solutions.bitbadger.documents.query.Find
import kotlin.test.Test
import kotlin.test.assertEquals
import kotlin.test.assertNotNull
import kotlin.test.assertNull

/**
 * SQLite integration tests for the `Custom` object / `custom*` connection extension functions
 */
@DisplayName("SQLite - Custom")
class CustomSQLiteIT {

    @Test
    @DisplayName("list succeeds with empty list")
    fun listEmpty() =
        SQLiteDB().use { db ->
            JsonDocument.load(db.conn, SQLiteDB.tableName)
            db.conn.customNonQuery("DELETE FROM ${SQLiteDB.tableName}")
            val result = db.conn.customList<JsonDocument>(Find.all(SQLiteDB.tableName), mapFunc = Results::fromData)
            assertEquals(0, result.size, "There should have been no results")
        }

    @Test
    @DisplayName("list succeeds with a non-empty list")
    fun listAll() =
        SQLiteDB().use { db ->
            JsonDocument.load(db.conn, SQLiteDB.tableName)
            val result = db.conn.customList<JsonDocument>(Find.all(SQLiteDB.tableName), mapFunc = Results::fromData)
            assertEquals(5, result.size, "There should have been 5 results")
        }

    @Test
    @DisplayName("single succeeds when document not found")
    fun singleNone() =
        SQLiteDB().use { db ->
            assertNull(
                db.conn.customSingle(Find.all(SQLiteDB.tableName), mapFunc = Results::fromData),
                "There should not have been a document returned"
            )
        }

    @Test
    @DisplayName("single succeeds when a document is found")
    fun singleOne() {
        SQLiteDB().use { db ->
            JsonDocument.load(db.conn, SQLiteDB.tableName)
            assertNotNull(
                db.conn.customSingle<JsonDocument>(Find.all(SQLiteDB.tableName), mapFunc = Results::fromData),
                "There should not have been a document returned"
            )
        }
    }

    @Test
    @DisplayName("nonQuery makes changes")
    fun nonQueryChanges() =
        SQLiteDB().use { db ->
            JsonDocument.load(db.conn, SQLiteDB.tableName)
            assertEquals(
                5L, db.conn.customScalar(Count.all(SQLiteDB.tableName), mapFunc = Results::toCount),
                "There should have been 5 documents in the table"
            )
            db.conn.customNonQuery("DELETE FROM ${SQLiteDB.tableName}")
            assertEquals(
                0L, db.conn.customScalar(Count.all(SQLiteDB.tableName), mapFunc = Results::toCount),
                "There should have been no documents in the table"
            )
        }

    @Test
    @DisplayName("nonQuery makes no changes when where clause matches nothing")
    fun nonQueryNoChanges() =
        SQLiteDB().use { db ->
            JsonDocument.load(db.conn, SQLiteDB.tableName)
            assertEquals(
                5L, db.conn.customScalar(Count.all(SQLiteDB.tableName), mapFunc = Results::toCount),
                "There should have been 5 documents in the table"
            )
            db.conn.customNonQuery(
                "DELETE FROM ${SQLiteDB.tableName} WHERE data->>'id' = :id",
                listOf(Parameter(":id", ParameterType.STRING, "eighty-two"))
            )
            assertEquals(
                5L, db.conn.customScalar(Count.all(SQLiteDB.tableName), mapFunc = Results::toCount),
                "There should still have been 5 documents in the table"
            )
        }

    @Test
    @DisplayName("scalar succeeds")
    fun scalar() =
        SQLiteDB().use { db ->
            assertEquals(
                3L,
                db.conn.customScalar("SELECT 3 AS it FROM ${SQLiteDB.catalog} LIMIT 1", mapFunc = Results::toCount),
                "The number 3 should have been returned"
            )
        }
}
