package solutions.bitbadger.documents

import org.junit.jupiter.api.DisplayName
import kotlin.test.Test
import kotlin.test.assertEquals
import kotlin.test.fail

/**
 * SQLite integration tests for the `Document` object / `insert`, `save`, `update` connection extension functions
 */
@DisplayName("SQLite - Document")
class DocumentSQLiteIT {

    @Test
    @DisplayName("insert works with default values")
    fun insertDefault() =
        SQLiteDB().use { db ->
            assertEquals(0L, db.conn.countAll(SQLiteDB.tableName), "There should be no documents in the table")
            val doc = JsonDocument("turkey", "", 0, SubDocument("gobble", "gobble"))
            db.conn.insert(SQLiteDB.tableName, doc)
            val after = db.conn.findAll<JsonDocument>(SQLiteDB.tableName)
            assertEquals(1, after.size, "There should be one document in the table")
            assertEquals(doc, after[0], "The document should be what was inserted")
        }

    @Test
    @DisplayName("insert fails with duplicate key")
    fun insertDupe() =
        SQLiteDB().use { db ->
            db.conn.insert(SQLiteDB.tableName, JsonDocument("a", "", 0, null))
            try {
                db.conn.insert(SQLiteDB.tableName, JsonDocument("a", "b", 22, null))
                fail("Inserting a document with a duplicate key should have thrown an exception")
            } catch (_: Exception) {
                // yay
            }
        }

    @Test
    @DisplayName("insert succeeds with numeric auto IDs")
    fun insertNumAutoId() =
        SQLiteDB().use { db ->
            try {
                Configuration.autoIdStrategy = AutoId.NUMBER
                Configuration.idField = "key"
                assertEquals(0L, db.conn.countAll(SQLiteDB.tableName), "There should be no documents in the table")

                db.conn.insert(SQLiteDB.tableName, NumIdDocument(0, "one"))
                db.conn.insert(SQLiteDB.tableName, NumIdDocument(0, "two"))
                db.conn.insert(SQLiteDB.tableName, NumIdDocument(77, "three"))
                db.conn.insert(SQLiteDB.tableName, NumIdDocument(0, "four"))

                val after = db.conn.findAll<NumIdDocument>(SQLiteDB.tableName, listOf(Field.named("key")))
                assertEquals(4, after.size, "There should have been 4 documents returned")
                assertEquals(
                    "1|2|77|78", after.joinToString("|") { it.key.toString() },
                    "The IDs were not generated correctly"
                )
            } finally {
                Configuration.autoIdStrategy = AutoId.DISABLED
                Configuration.idField = "id"
            }
        }

    // TODO: UUID, Random String
}
